package javax.swing.text.html;

import java.util.NoSuchElementException;
import java.util.Enumeration;
import javax.swing.text.SimpleAttributeSet;
import java.io.Serializable;
import javax.swing.text.AttributeSet;

class MuxingAttributeSet implements AttributeSet, Serializable
{
    private AttributeSet[] attrs;
    
    public MuxingAttributeSet(AttributeSet[] attrs) {
        this.attrs = attrs;
    }
    
    protected MuxingAttributeSet() {
    }
    
    protected synchronized void setAttributes(AttributeSet[] attrs) {
        this.attrs = attrs;
    }
    
    protected synchronized AttributeSet[] getAttributes() {
        return this.attrs;
    }
    
    protected synchronized void insertAttributeSetAt(AttributeSet set, int n) {
        int length = this.attrs.length;
        AttributeSet[] attrs = new AttributeSet[length + 1];
        if (n < length) {
            if (n > 0) {
                System.arraycopy(this.attrs, 0, attrs, 0, n);
                System.arraycopy(this.attrs, n, attrs, n + 1, length - n);
            }
            else {
                System.arraycopy(this.attrs, 0, attrs, 1, length);
            }
        }
        else {
            System.arraycopy(this.attrs, 0, attrs, 0, length);
        }
        attrs[n] = set;
        this.attrs = attrs;
    }
    
    protected synchronized void removeAttributeSetAt(int n) {
        int length = this.attrs.length;
        AttributeSet[] attrs = new AttributeSet[length - 1];
        if (length > 0) {
            if (n == 0) {
                System.arraycopy(this.attrs, 1, attrs, 0, length - 1);
            }
            else if (n < length - 1) {
                System.arraycopy(this.attrs, 0, attrs, 0, n);
                System.arraycopy(this.attrs, n + 1, attrs, n, length - n - 1);
            }
            else {
                System.arraycopy(this.attrs, 0, attrs, 0, length - 1);
            }
        }
        this.attrs = attrs;
    }
    
    @Override
    public int getAttributeCount() {
        AttributeSet[] attributes = this.getAttributes();
        int n = 0;
        for (int i = 0; i < attributes.length; ++i) {
            n += attributes[i].getAttributeCount();
        }
        return n;
    }
    
    @Override
    public boolean isDefined(Object o) {
        AttributeSet[] attributes = this.getAttributes();
        int i = 0;
        while (i < attributes.length) {
            if (attributes[i].isDefined(o)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public boolean isEqual(AttributeSet set) {
        return this.getAttributeCount() == set.getAttributeCount() && this.containsAttributes(set);
    }
    
    @Override
    public AttributeSet copyAttributes() {
        AttributeSet[] attributes = this.getAttributes();
        SimpleAttributeSet set = new SimpleAttributeSet();
        for (int i = attributes.length - 1; i >= 0; --i) {
            set.addAttributes(attributes[i]);
        }
        return set;
    }
    
    @Override
    public Object getAttribute(Object o) {
        AttributeSet[] attributes = this.getAttributes();
        int length = attributes.length;
        int i = 0;
        while (i < length) {
            Object attribute = attributes[i].getAttribute(o);
            if (attribute != null) {
                return attribute;
            }
            ++i;
        }
        return null;
    }
    
    @Override
    public Enumeration getAttributeNames() {
        return new MuxingAttributeSet.MuxingAttributeNameEnumeration();
    }
    
    @Override
    public boolean containsAttribute(Object o, Object o2) {
        return o2.equals(this.getAttribute(o));
    }
    
    @Override
    public boolean containsAttributes(AttributeSet set) {
        boolean equals = true;
        Object nextElement;
        for (Enumeration attributeNames = set.getAttributeNames(); equals && attributeNames.hasMoreElements(); equals = set.getAttribute(nextElement).equals(this.getAttribute(nextElement))) {
            nextElement = attributeNames.nextElement();
        }
        return equals;
    }
    
    @Override
    public AttributeSet getResolveParent() {
        return null;
    }
    
    private class MuxingAttributeNameEnumeration implements Enumeration
    {
        private int attrIndex;
        private Enumeration currentEnum;
        
        MuxingAttributeNameEnumeration() {
            this.updateEnum();
        }
        
        @Override
        public boolean hasMoreElements() {
            return this.currentEnum != null && this.currentEnum.hasMoreElements();
        }
        
        @Override
        public Object nextElement() {
            if (this.currentEnum == null) {
                throw new NoSuchElementException("No more names");
            }
            Object nextElement = this.currentEnum.nextElement();
            if (!this.currentEnum.hasMoreElements()) {
                this.updateEnum();
                return nextElement;
            }
            return nextElement;
        }
        
        void updateEnum() {
            AttributeSet[] attributes = MuxingAttributeSet.this.getAttributes();
            this.currentEnum = null;
            while (this.currentEnum == null && this.attrIndex < attributes.length) {
                this.currentEnum = attributes[this.attrIndex++].getAttributeNames();
                if (this.currentEnum.hasMoreElements()) {
                    continue;
                }
                this.currentEnum = null;
            }
        }
    }
}
